package provide DS9 1.0

# Public Functions

proc CreateColorbar {} {
    global colorbar
    global ds9

    set colorbar(name) \
	[canvas $ds9(main).cmap -width $colorbar(width) \
	     -height $colorbar(height) -bd 2 -relief groove -insertofftime 0]
    switch -- $ds9(visual) {
	pseudocolor {
	    $colorbar(name) create colorbar$ds9(visual)$ds9(depth) \
		-private $colorbar(private) \
		-width $colorbar(width) -height $colorbar(height) -anchor nw

	    # And set the two windows that where created before the 
	    # colorbar was created, to the new colormap

	    colorbar set colormap window "$ds9(main)"

	    # This is a kluge, but it works. We want to reassign the bg color 
	    # for $ds9(main) from the old colormap to the new colormap
	    
	    $ds9(main) configure -background [$ds9(main) cget -background]
	}
	truecolor {
	    $colorbar(name) create colorbar$ds9(visual)$ds9(depth) \
		-width $colorbar(width) -height $colorbar(height) \
		-anchor nw -colors 2048
	    $colorbar(name) create colorbarrgb$ds9(visual)$ds9(depth) \
		-width $colorbar(width) -height $colorbar(height) \
		-anchor nw -colors 2048
	}
    }
}

proc InitColorbar {} {
    global current
    global colorbar

    set current(colorbar) colorbar

    colorbar map "{$colorbar(map)}"
    colorbar invert $colorbar(invert)
    bind $colorbar(name) <Configure> [list UpdateColorbarGeometry]
}

proc ResetColormap {} {
    global current

    $current(colorbar) reset
    if {$current(frame) != "" } {
	$current(frame) colormap [$current(colorbar) get colormap]
	set colorbar(invert) [$current(colorbar) get invert]
    }

    UpdateColormapDialog
}

proc LoadColormap {} {
    LoadColormapFile [OpenFileDialog colorbarfbox]
}

proc LoadColormapFile {filename} {
    global current
    global colorbar
    global ds9

    if {$filename != {}} {
	colorbar load "\{$filename\}"
	set id [colorbar get id]
	set colorbar(map) [colorbar get name]
	$ds9(mb).color insert $colorbar(count) radiobutton \
	    -label "$colorbar(map)" \
	    -variable colorbar(map) -command "ChangeColormapID $id"
	incr colorbar(count)

	ChangeColormapID $id
    }
}

proc SaveColormap {} {
    global ds9
    global current

    FileLast colorbarfbox [colorbar get file name]
    set filename [SaveFileDialog colorbarfbox]
    if {$filename != {}} {
	colorbar save "\{$filename\}"
    }
}

proc ButtonColormap {x y} {
    global current

    if {$current(frame) != ""} {
	$current(frame) colormap begin $x $y
    }
}

proc MotionColormap {x y} {
    global canvas
    global current
    global colorbar

    # X sets bias
    set cursorX \
	[expr double($x-$canvas(width)/2+$colorbar(speed)/2)/$colorbar(speed)]
    # Y set contrast
    set cursorY [expr double($y)/$colorbar(speed) * $colorbar(mult)]

    $current(colorbar) adjust $cursorY $cursorX
    if {$current(frame) != ""} {
	$current(frame) colormap motion [$current(colorbar) get colormap]
    }

    UpdateColormapDialog
}

proc ReleaseColormap {x y} {
    global current

    if {$current(frame) != ""} {
	$current(frame) colormap end [$current(colorbar) get colormap]
    }

    UpdateColormapDialog
}

proc CreateColorMenus {} {
    global current
    global colorbar
    global ds9

    set id [colorbar list id]

    foreach i $id {
	set name [colorbar get name $i]
	$ds9(mb).color insert $colorbar(count) radiobutton \
	    -label "$name" -variable colorbar(map) \
	    -command "ChangeColormapID $i"
	$ds9(mb).prefs.color insert [expr $colorbar(count)-1] radiobutton \
	    -label "$name" -variable prefs(colorbar,map)
	incr colorbar(count)
    }
}

proc UpdateColorbarGeometry {} {
    global ds9
    global current
    global colorbar

    set colorbar(width) [expr [winfo width $colorbar(name)]-2]
    set colorbar(height) [expr [winfo height $colorbar(name)]-2]

    switch -- $ds9(visual) {
	pseudocolor {
	    colorbar configure -width $colorbar(width) \
		-height $colorbar(height)
	}
	truecolor {
	    colorbar configure -width $colorbar(width) \
		-height $colorbar(height)
	    colorbarrgb configure -width $colorbar(width) \
		-height $colorbar(height)
	}
    }
}

proc ChangeColormapID {id} {
    global current
    global colorbar
    
    SetWatchCursor
    colorbar map $id
    if {$current(frame) != "" } {
	$current(frame) colormap [colorbar get colormap]
	set colorbar(map) [colorbar get name]
	set colorbar(invert) [colorbar get invert]
    }
    UnsetWatchCursor

    UpdateColormapDialog
}

proc ChangeColormap {} {
    global current
    global colorbar
    
    SetWatchCursor
    colorbar map "{$colorbar(map)}"
    if {$current(frame) != "" } {
	$current(frame) colormap [colorbar get colormap]
	set colorbar(invert) [colorbar get invert]
    }
    UnsetWatchCursor

    UpdateColormapDialog
}

proc MatchColorbars {} {
    global ds9
    global current
    global colorbar

    SetWatchCursor
    set id [$current(colorbar) get colormap]
    foreach f $ds9(frames) {
	if {$f!=$current(frame) && [$f get type]==[$current(frame) get type]} {
	    $f colormap $id
	}
    }
    UnsetWatchCursor
}

proc InvertColorbar {} {
    global current
    global colorbar

    SetWatchCursor
    colorbar invert $colorbar(invert)
    if {$current(frame) != "" } {
	$current(frame) colormap [colorbar get colormap]
    }
    UnsetWatchCursor

    UpdateColormapDialog
}

proc ColorbarArea {} {
    global ds9
    global colorbar

    foreach f $ds9(frames) {
	$f colormap area $colorbar(area) $colorbar(mode)
    }
}

proc ColormapDialog {} {
    global colorbar
    global ds9

    # see if we already have a window visible

    if [winfo exist $colorbar(top)] {
	raise $colorbar(top)
	return
    }

    set w $colorbar(top)
    set title "Contrast/Bias"
    set length 300

    # create the window

    toplevel $w -colormap $ds9(main)
    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW ColormapDestroyDialog

    UpdateColormapDialog

    frame $w.cb  -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.cb $w.buttons -ipadx 4 -ipady 4 -fill x -expand true

    frame $w.cb.contrast
    scale $w.cb.contrast.slider -from 0 -to 10 -length $length \
	-variable colorbar(contrast) -orient horizontal -label "Contrast" \
	-tickinterval 2 -showvalue false -resolution .1 \
	-command AdjustColormap
    entry $w.cb.contrast.value -textvariable colorbar(contrast) -width 7 \
	-relief sunken -bd 2
    pack $w.cb.contrast.slider $w.cb.contrast.value -side left -padx 4
    pack $w.cb.contrast.value

    frame $w.cb.bias
    scale $w.cb.bias.slider -from 0 -to 1 -length $length \
	-variable colorbar(bias) -orient horizontal -label "Bias" \
	-tickinterval .2 -showvalue false -resolution .01 \
	-command AdjustColormap
    entry $w.cb.bias.value -textvariable colorbar(bias) -width 7 \
	-relief sunken -bd 2
    pack $w.cb.bias.slider $w.cb.bias.value -side left -padx 4
    pack $w.cb.bias.value

    pack $w.cb.contrast $w.cb.bias -side top

    button $w.buttons.apply -text "Apply" -command ColormapApplyDialog
    button $w.buttons.close -text "Close" -command ColormapDestroyDialog
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true

    bind $w.cb.contrast.slider <Button-1> BeginAdjustColormap
    bind $w.cb.contrast.slider <ButtonRelease-1> EndAdjustColormap
    bind $w.cb.bias.slider <Button-1> BeginAdjustColormap
    bind $w.cb.bias.slider <ButtonRelease-1> EndAdjustColormap
}

proc ColormapDestroyDialog {} {
    global colorbar

    destroy $colorbar(top)

    unset colorbar(bias)
    unset colorbar(contrast)
}

proc ColormapApplyDialog {} {
    BeginAdjustColormap
    AdjustColormap 1
    EndAdjustColormap
}

proc BeginAdjustColormap {} {
    global current
    global colorbar

    set colorbar(adjustok) 1
    if {$current(frame) != ""} {
	$current(frame) colormap begin
    }
}

proc AdjustColormap {value} {
    # value is ignored
    global current
    global colorbar

    if {[info exists colorbar(adjustok)]} {
	$current(colorbar) adjust $colorbar(contrast) $colorbar(bias)
	if {$current(frame) != ""} {
	    $current(frame) colormap motion [$current(colorbar) get colormap]
	}
    }
}

proc EndAdjustColormap {} {
    global current
    global colorbar

    if {[info exists colorbar(adjustok)]} {
	unset colorbar(adjustok)
	if {$current(frame) != ""} {
	    $current(frame) colormap end [$current(colorbar) get colormap]
	}
    }
}

proc UpdateColormapMenu {} {
    global ds9
    global current
    global menu
    global buttons

    global debug
    if {$debug(tcl,update)} {
	puts "UpdateColormapMenu"
    }

    if {$current(frame) != ""} {
	if {[$current(frame) get type] == "rgb"} {
	    $ds9(mb).color entryconfig "Invert Colormap" -state disabled
	    $ds9(mb).color entryconfig "Load Colormap..." -state disabled
	    $ds9(mb).color entryconfig "Save Colormap..." -state disabled
	    for {set i 1} {$i<=18} {incr i} {
		$ds9(mb).color entryconfig $i -state disabled
	    }

	    $buttons(name).color1.invert configure -state disabled
	    $buttons(name).color1.load configure -state disabled
	    $buttons(name).color1.save configure -state disabled
	    for {set i 0} {$i<9} {incr i} {
		$buttons(name).color2.$i configure -state disabled
	    }
	    for {set i 9} {$i<18} {incr i} {
		$buttons(name).color3.$i configure -state disabled
	    }

	} else {
	    $ds9(mb).color entryconfig "Invert Colormap" -state normal
	    $ds9(mb).color entryconfig "Load Colormap..." -state normal
	    $ds9(mb).color entryconfig "Save Colormap..." -state normal
	    for {set i 1} {$i<=18} {incr i} {
		$ds9(mb).color entryconfig $i -state normal
	    }

	    $buttons(name).color1.invert configure -state normal
	    $buttons(name).color1.load configure -state normal
	    $buttons(name).color1.save configure -state normal
	    for {set i 0} {$i<9} {incr i} {
		$buttons(name).color2.$i configure -state normal
	    }
	    for {set i 9} {$i<18} {incr i} {
		$buttons(name).color3.$i configure -state normal
	    }
	}
    } else {
	$ds9(mb).color entryconfig "Invert Colormap" -state normal
	$ds9(mb).color entryconfig "Load Colormap..." -state normal
	$ds9(mb).color entryconfig "Save Colormap..." -state normal
	for {set i 1} {$i<=18} {incr i} {
	    $ds9(mb).color entryconfig $i -state normal
	}

	$buttons(name).color1.invert configure -state normal
	$buttons(name).color1.load configure -state normal
	$buttons(name).color1.save configure -state normal
	for {set i 0} {$i<9} {incr i} {
	    $buttons(name).color2.$i configure -state normal
	}
	for {set i 9} {$i<18} {incr i} {
	    $buttons(name).color3.$i configure -state normal
	}
    }
}

proc UpdateColormapDialog {} {
    global current
    global colorbar

    global debug
    if {$debug(tcl,update)} {
	puts "UpdateColormapDialog"
    }

    if {[winfo exist $colorbar(top)]} {
	set colorbar(contrast) [$current(colorbar) get contrast]
	set colorbar(bias) [$current(colorbar) get bias]
    }
}

proc ProcessCmapCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global ds9
    global current
    global colorbar

    switch -- [string tolower [lindex $var $i]] {
	file {
	    incr i
	    LoadColormapFile [lindex $var $i]
	    FileLast colormapfbox [lindex $var $i]
	}
	invert {
	    incr i
	    set colorbar(invert) [FromYesNo [lindex $var $i]]
	    InvertColorbar
	}
	value {
	    incr i
	    set c [lindex $var $i]
	    incr i
	    set b [lindex $var $i]
	    $current(colorbar) adjust $c $b
	    if {$current(frame) != ""} {
		$current(frame) colormap begin
		$current(frame) colormap motion \
		    [$current(colorbar) get colormap]
		$current(frame) colormap end \
		    [$current(colorbar) get colormap]
	    }
	}
	default {
	    set colorbar(map) [lindex $var $i]
	    ChangeColormap
	}
    }
    UpdateColormapDialog
}
