package provide DS9 1.0

proc ProcessHelp {} {
    global argc
    global argv

    if {$argc > 0 && ([lindex $argv 0] == "-?")} {
	DumpHelp
	exit
    }
}

proc BadVisualError {} {
    global ds9

    tk_messageBox -parent $ds9(top) -icon info -type ok -message \
	"Sorry, DS9 requires a Pseudocolor8, Truecolor8, Truecolor16, Truecolor24 visual be available."
    exit
}

proc ProcessCommandLineFirst {} {
    global argc
    global argv
    global colorbar
    global ds9

    set i 0
    while {$i < $argc} {
	set item [lindex $argv $i]
	switch -- $item {
	    -debug {incr i; ProcessDebugTclCmd argv i}
	    -private {set colorbar(private) true}
	    -title {
		incr i
		set ds9(title) [lindex $argv $i]
		set t "SAOImage $ds9(title)"
		wm title $ds9(top) "$t"
		wm iconname $ds9(top) "$t"
	    }
	}
	incr i
    }
}

proc ProcessCommandLineXPA {} {
    global argc
    global argv
    global xpa

    set i 0
    while {$i < $argc} {
	set item [lindex $argv $i]
	switch -- $item {
	    -vo {incr i; ProcessVOCmd argv i}
	}
	incr i
    }
}

proc ProcessCommandLine {} {
    global argc
    global argv
    global tcl_platform

    global ds9
    global help
    global current
    global contour
    global colorbar
    global debug
    global wcs
    global info
    global pan
    global mag
    global grid
    global graph
    global graphHorz
    global graphVert
    global bin
    global scale
    global iis
    global minmax
    global tcl_platform
    global http

    set frameCount 0
    set load 0
    set file(mode) file
    set i 0

    # Note: help is processed previously
    while {$i < $argc} {
	set item [lindex $argv $i]
	switch -- $item {
	    -about {
		if {$tcl_platform(platform)!= "windows"} {
		    puts "$help(about)"
		}
	    }
	    -align {#back; set wcs(align) 1; AlignWCSFrame}
	    -analysis -
	    -analysismenu {
		incr i
		set item [lindex $argv $i]
		ProcessAnalysisFile $item
		FileLast analysisfbox $item
		LayoutFrames
	    }
	    -array {set file(mode) array}
	    -bin {incr i; ProcessBinCmd argv i}
	    -blink {incr i; ProcessBlinkCmd argv i}
	    -blue {set current(rgb) blue; RGBChannel}
	    -cmap {incr i; ProcessCmapCmd argv i}
	    -colorbar {#back; set colorbar(view) 1; UpdateView}
	    -nocolorbar {#back; set colorbar(view) 0; UpdateView}
	    -contour {incr i; ProcessContourCmd argv i}
	    -nocontour {set contour(view) 0; ContourUpdate}
	    -datacube {incr i; ProcessDataCubeCmd argv i}
	    -debug {incr i; ProcessDebugCmd argv i}
	    -dss {incr i; ProcessDSSCmd argv i 1 frameCount}
	    -exit {QuitDS9}
	    -quit {QuitDS9}
	    -frame {incr i; ProcessFrameCmd argv i}
	    -fifo {
		incr i
		set fifo [lindex $argv $i]
		if {$fifo!="none"} {
		    set iis(ififo) ${fifo}i
		    set iis(ofifo) ${fifo}o
		} else {
		    set iis(ififo) none
		    set iis(ofifo) none
		}
	    }
	    -fifo_only {set iis(port) 0; set iis(unix) none}
	    -file {set file(mode) file}
	    -geometry {#already processed}
	    -green {set current(rgb) green; RGBChannel}
	    -grid {incr i; ProcessGridCmd argv i}
	    -nogrid {set grid(view) 0; GridUpdate}
	    -help {
		if {$tcl_platform(platform)!= "windows"} {
		    DumpHelp
		    exit
		}
	    }
	    -histequ {set scale(type) histequ; ChangeScale}
	    -horzgraph {#back; set graphHorz(view) 1; UpdateView}
	    -nohorzgraph {#back; set graphHorz(view) 0; UpdateView}
	    -info {#back; set info(view) 1; UpdateView}
	    -noinfo {#back; set info(view) 0; UpdateView}
	    -iconify {wm iconify $ds9(top)}
	    -invert {set colorbar(invert) 1; InvertColorbar}
	    -lock {incr i; ProcessLockCmd argv i}
	    -linear {set scale(type) linear; ChangeScale}
	    -log {set scale(type) log; ChangeScale}
	    -lower {lower $ds9(top)}
	    -magnifier {#back; set mag(view) 1; UpdateView}
	    -nomagnifier {#back; set mag(view) 0; UpdateView}
	    -match {incr i; ProcessMatchCmd argv i}
	    -medatacube {set file(mode) medatacube}
	    -minmax {incr i; ProcessMinMaxCmd argv i}
	    -minmaxmode {#back; incr i; set minmax(mode) [lindex $argv $i]}
	    -minmaxsample {#back; incr i; set minmax(sample) [lindex $argv $i]}
	    -mode {incr i; set ds9(mode) [lindex $argv $i]; ChangeMode}
	    -mosaicimage -
	    -mosaicimageiraf {set file(mode) mosaicimageiraf}
	    -mosaic -
	    -mosaiciraf {set file(mode) mosaiciraf}
	    -mosaicimagewcs {set file(mode) mosaicimagewcs}
	    -mosaicwcs {set file(mode) mosaicwcs}
	    -mosaicwfpc2 -
	    -mosaicimagewfpc2 {set file(mode) mosaicimagewfpc2}
	    -nameserver {incr i; ProcessNSVRCmd argv i}
	    -orient {incr i; ProcessOrientCmd argv i}
	    -pagesetup {incr i; ProcessPageSetupCmd argv i}
	    -pan {RealizeDS9; incr i; ProcessPanCmd argv i}
	    -panner {#back; set pan(view) 1; UpdateView}
	    -nopanner {#back; set pan(view) 0; UpdateView}
	    -pixeltable {PixelTableDisplay}
	    -nopixeltable {PixelTableDestroy}
	    -plot {incr i; set buf {}; ProcessPlotCmd argv i buf}
	    -port {incr i; set iis(port) [lindex $argv $i]}
	    -inet_only -
	    -port_only {
		set iis(ififo) none
		set iis(ofifo) none
		set iis(unix) none
	    }
	    -prefs {incr i; ProcessPrefs argv i}
	    -print {incr i; ProcessPrintCmd argv i}
	    -private {#already processed}
	    -raise {raise $ds9(top)}
	    -red {set current(rgb) red; RGBChannel}
	    -region -
	    -regionfile {
		RealizeDS9
		incr i
		LoadMarker fg [lindex $argv $i]
		FileLast markerfbox [lindex $argv $i]
	    }
	    -rgb {incr i; ProcessRGBCmd argv i}
	    -rgbcube {set file(mode) rgbcube}
	    -rgbimage {set file(mode) rgbimage}
	    -rgbarray {set file(mode) rgbarray}
	    -rotate {incr i; ProcessRotateCmd argv i}
	    -saveas {incr i; ProcessSaveAsCmd argv i}
	    -scale -
	    -ztrans {incr i; ProcessScaleCmd argv i}
	    -scalelims -
	    -scalelimits {
		#backward compatibility
		incr i
		set scale(usermin) [lindex $argv $i]
		incr i
		set scale(usermax) [lindex $argv $i]
		ChangeScaleLimit
	    }
	    -scalemode {
		#backward compatibility
		incr i
		set scale(mode) [string tolower [lindex $argv $i]]
		ChangeScaleMode
	    }
	    -scalescope {
		#backward compatibility
		incr i
		set scale(scope) [string tolower [lindex $argv $i]]
		ChangeScaleScope
	    }
	    -shm {incr i; ProcessShmCmd argv i 1 frameCount}
	    -single {incr i; ProcessSingleCmd argv i}
	    -squared {set scale(type) squared; ChangeScale}
	    -sqrt {set scale(type) sqrt; ChangeScale}
	    -source {
		incr i
		uplevel #0 "source \{[lindex $argv $i]\}"
	    }
	    -tile {incr i; ProcessTileCmd argv i}
	    -title {incr i; #already processed}
	    -unix {incr i; set iis(unix) [lindex $argv $i]}
	    -unix_only {
		set iis(ififo) none
		set iis(ofifo) none
		set iis(port) 0
	    }
	    -url {set file(mode) url}
	    -version {
		if {$tcl_platform(platform)!= "windows"} {
		    puts "$ds9(version)"
		}
	    }
	    -vertgraph {#back; set graphVert(view) 1; UpdateView}
	    -novertgraph {#back; set graphVert(view) 0; UpdateView}
	    -coords -
	    -view {incr i; ProcessViewCmd argv i}
	    -visual {#already processed}
	    -vo {incr i; #will process later}
	    -wcs {incr i; ProcessWCSCmd argv i}
	    -wcsformat {#back; incr i; set wcs(format,) [lindex $argv $i]}
	    -web {incr i; ProcessWebCmd argv i}
	    -xpa {incr i; ProcessXPACmd argv i}
	    -z1 {
		#backward compatibility
		incr i
		set scale(usermin) [lindex $argv $i]
		ChangeScaleLimit
	    }
	    -z2 {
		#backward compatibility
		incr i
		set scale(usermax) [lindex $argv $i]
		ChangeScaleLimit
	    }
	    -zmax {set scale(mode) zmax; ChangeScaleMode}
	    -zscale {set scale(mode) zscale; ChangeScaleMode}
	    -zoom {incr i; ProcessZoomCmd argv i}

	    default {
		if {([string length $item] == 1) || \
		        ([string range $item 0 1] == {-[}) || \
			([string range $item 0 0] != "-")} {

		    if {$load == 0} {
		        StartLoad
		        incr load
		    }

		    # we could have a 'dos' style file name, so convert
		    if {$tcl_platform(platform) == "windows" &&
		        $item != "-"} {
			set item [$current(frame) convert path $item]
		    }

		    switch [$current(frame) get type] {
			base {
			    switch $file(mode) {
				array {
				    MultiLoad frameCount
				    LoadArray $item
				    FileLast arrayfbox $item
				}
				file {
				    MultiLoad frameCount
				    LoadFits $item
				    FileLast fitsfbox $item
				}
				medatacube {
				    MultiLoad frameCount
				    LoadDataCubeFits $item
				    FileLast fitsfbox $item
				}
				mosaicimageiraf {
				    MultiLoad frameCount
				    LoadMosaicImageIRAFFits $item
				    FileLast fitsfbox $item
				}
				mosaiciraf {
				    LoadMosaicIRAFFits $item
				    FileLast fitsfbox $item
				}
				mosaicimagewcs {
				    MultiLoad frameCount
				    LoadMosaicImageWCSFits $item
				    FileLast fitsfbox $item
				}
				mosaicwcs {
				    LoadMosaicWCSFits $item
				    FileLast fitsfbox $item
				}
				mosaicimagewfpc2 {
				    MultiLoad frameCount
				    LoadMosaicImageWFPC2Fits $item
				    FileLast fitsfbox $item
				}
				rgbcube {
				    MultiRGBLoad frameCount
				    LoadRGBCubeFits $item
				    FileLast fitsfbox $item
				}
				rgbimage {
				    MultiRGBLoad frameCount
				    LoadRGBImageFits $item
				    FileLast fitsfbox $item
				}
				rgbarray {
				    MultiRGBLoad frameCount
				    LoadRGBArray $item
				    FileLast arrayfbox $item
				}
				url {
				    MultiLoad frameCount
				    LoadURL $item
				    set http(last) $item
				}
			    }
			}
			rgb {
			    switch $file(mode) {
				array {
				    LoadArray $item
				    FileLast arrayfbox $item
				}
				file {
				    LoadFits $item
				    FileLast fitsfbox $item
				}
				medatacube {
				    LoadDataCubeFits $item
				    FileLast fitsfbox $item
				}
				mosaic {
				    LoadMosaicFits $item
				    FileLast fitsfbox $item
				}
				mosaicimage {
				    LoadMosaicImageFits $item
				    FileLast fitsfbox $item
				}
				mosaicwfpc2 {
				    LoadMosaicImageWFPC2Fits $item
				    FileLast fitsfbox $item
				}
				rgbcube {
				    MultiRGBLoad frameCount
				    LoadRGBCubeFits $item
				    FileLast fitsfbox $item
				}
				rgbimage {
				    MultiRGBLoad frameCount
				    LoadRGBImageFits $item
				    FileLast fitsfbox $item
				}
				rgbarray {
				    MultiRGBLoad frameCount
				    LoadRGBArray $item
				    FileLast arrayfbox $item
				}
				url {
				    LoadURL $item
				    set http(last) $item
				}
			    }
			}
		    }
		    GridUpdate
		    ContourUpdate

		} else {
		    if {$tcl_platform(platform)!= "windows"} {
			puts "Unknown command line option: $item"
			DumpHelp
			exit
		    }
		}
	    }
	}
	incr i
    }

    if {$load != 0} {
        FinishLoad
    }
}

