#!/bin/sh
# the next line restarts using wish
# exec wish8.4 "$0" "$@"

# debug level
set debug(tcl,events) 0
set debug(tcl,update) 0
set debug(tcl,idletasks) 0
set debug(tcl,layout) 0
set debug(tcl,info) 0
set debug(tcl,watch) 0
set debug(tcl,hv) 0
set debug(tcl,grid) 0
set debug(tcl,http) 0
set debug(tcl,ftp) 0

set debug(saotk,ast) 0
set debug(saotk,mosaic) 0
set debug(saotk,parser) 0
set debug(saotk,perf) 0
set debug(saotk,wcs) 0
set debug(saotk,bin) 0
set debug(saotk,gz) 0
set debug(saotk,rgb) 0

set debug(iis) 0

# we don't want to see any error messages if xpa is not available
set env(XPA_VERBOSITY) 0

# source other scripts
# we do it this way because mktclapp does not implement package require
source source.tcl

# required packages
package require http

package require Ast
package require BLT
package require checkdns
package require saotk
package require Tkhtml
package require tclxpa
package require iis
package require img::base
package require img::jpeg
package require img::tiff
package require img::png
package require img::gif
package require img::ppm
package require img::bmp
package require img::xbm

# we need this after BLT is loaded
# neede for BLT_ZoomStack
source ../lib/blt2.4/graph.tcl

# let's start
set ds9(init) 1

# Set Application Name
if {$tcl_platform(platform) != "windows"} {
    if {"$argv0"=="ds9.tcl"} {
	set ds9(app) ds9
    } else {
	set ds9(app) [file tail $argv0]
    }
} else {
    set ds9(app) ds9
}

tk appname $ds9(app)
set ds9(title) "$ds9(app)"
set ds9(version) "3.0b6"

# First, look for help quest
InitHelp
ProcessHelp

# ds9 Variables
# General Note:
# ds9(frames) contains list of all frames
# ds9(active) contains list of active (show) frames
# active(<framename>) contains status show/hide for framename
# slave(<framename>) contains list of frames that are slaves to framename
# master(<framename>) contains the name of the master frame of framename

set ds9(top) .
set ds9(mb) .mb
set ds9(main) .ds9
set ds9(visual) ""
set ds9(depth) 8
set ds9(frames) ""
set ds9(active) ""
set ds9(lock) 0
set ds9(next) ""
set ds9(language) english
set ds9(mode) pointer
if {$tcl_platform(platform)!= "windows"} {
    set ds9(dialog) motif
} else {
    set ds9(dialog) windows
}
set ds9(display,mode) single
set ds9(display,user) single
set ds9(cursor) 1
set ds9(automarker) 1
set ds9(amp,det) 0

set ds9(xpa) 1
set ds9(xpa,tcl) 0
set ds9(xpa,ptr) 0
set ds9(xpa,idletasks) 1

set ds9(b1) 0
set ds9(b2) 0
set ds9(b3) 0

set ds9(sb1) 0
set ds9(sb2) 0
set ds9(sb3) 0

set ds9(cb1) 0
set ds9(cb2) 0
set ds9(cb3) 0

set ds9(csb1) 0
set ds9(csb2) 0
set ds9(csb3) 0

set ds9(internalpad) 2
set ds9(freeze) 0

# external analysis and file formats
set ds9(ext,file) ".$ds9(app).fil"
set ds9(ext,alt) ".$ds9(app).file"
set ds9(analysis,file) ".$ds9(app).ans"
set ds9(analysis,alt) ".$ds9(app).analysis"
set ds9(analysis,user) {}
set ds9(analysis,user2) {}
set ds9(analysis,user3) {}
set ds9(analysis,user4) {}

# temp directory
if {$tcl_platform(platform) != "windows"} {
    set ds9(tmpdir) "/tmp"
} else {
    set ds9(tmpdir) "."
}

set fitsfbox(file)      {}
set fitsfbox(directory) {}
set fitsfbox(dirslist)  {}
set fitsfbox(loadtype)  {}
set fitsfbox(pattern)   1
set fitsfbox(types) {
    {"Fits files"              *.fits}
    {"Fits files"              *.fts }
    {"Fits files"              *.fit }
    {"Compressed FITS files"   *.fitz }
    {"Compressed FITS files"   *.fits.gz }
    {"Compressed FITS files"   *.fits.Z }
    {"All files"               *}
}

set fitsavfbox(file)      {ds9.fits}
set fitsavfbox(directory) {}
set fitsavfbox(dirslist)  {}
set fitsavfbox(loadtype)  {}
set fitsavfbox(pattern)   0
set fitsavfbox(types) {
    {"Fits files"   *.fits}
    {"All files"    *}
}

set fitsvarfbox(file)      {}
set fitsvarfbox(directory) {}
set fitsvarfbox(dirslist)  {}
set fitsvarfbox(loadtype)  {}
set fitsvarfbox(pattern)   0
set fitsvarfbox(types) {
    {"Fits files"              *.fits }
    {"Fits files"              *.fts }
    {"Fits files"              *.fit }
    {"Compressed FITS files"   *.fitz }
    {"Compressed FITS files"   *.fits.gz }
    {"Compressed FITS files"   *.fits.Z }
    {"All files"               *}
}

set arrayfbox(file)      {}
set arrayfbox(directory) {}
set arrayfbox(dirslist)  {}
set arrayfbox(loadtype)  {}
set arrayfbox(pattern)   1
set arrayfbox(types) {
    {"Array files"   *.arr}
    {"All files"     *}
}

# internal file formats
set ds9(fileProcs) \
    {Stdin ExternalFits CompressFits PackFits GzipFits FTZFits}

if {$tcl_platform(platform) == "windows"} {
    if {[info exists env(TERM)]} {
	if {$env(TERM) != "cygwin"} {
	    set ds9(fileProcs) {}
	}
    } else {
	set ds9(fileProcs) {}
    }
}

# Pixel Table Variables
set pixel(top) .pixel
set pixel(mb) .pixelmb
set pixel(view) 0
set pixel(size) 5
set pixel(maxsize) 9
set pixel(copy) {}

set pixelfbox(file)      {ds9.pix}
set pixelfbox(directory) {}
set pixelfbox(dirslist)  {}
set pixelfbox(loadtype)  {}
set pixelfbox(pattern)   0
set pixelfbox(types) {
    {"Pixel Table files"   *.pix}
    {"All files"           *}
}

# Cursor Variables
set cursor(save) ""
set cursor(count) 0

# Crosshair Variables
set crosshair(lock) none

# Menu Variables
set menu(tearoff) 1
set menu(selectcolor) red

# Frame Variables
set current(frame) {}
set current(colorbar) {}
set current(cursor) {}

set current(zoom) 1
set current(rotate) 0
set current(orient) none

set current(rgb) red

blt::vector create histX histY

# RGB Variables
set rgb(red) 1
set rgb(green) 1
set rgb(blue) 1
set rgb(system) wcs
set rgb(lock,scale) 0
set rgb(lock,bin) 0

set rgb(top) .rgb
set rgb(mb) .rgbmb

# MinMax Variables
set minmax(mode) scan
set minmax(sample) 10

# Scale Variables
set scale(top) .scale
set scale(mb) .scalemb
set scale(type) linear
set scale(mode) minmax
set scale(scope) local
set scale(usermin) 1
set scale(usermax) 100
set scale(datasec) 1
set scale(xaxis) full
set scale(yaxis) log

# Group Variablables
set group(top) .grp
set group(mb) .grpmb

# Marker Variables
set marker(shape) circle
set marker(color) green
set marker(width) 1
set marker(edit) 1
set marker(move) 1
set marker(rotate) 1
set marker(delete) 1
set marker(fixed) 0
set marker(include) 1
set marker(source) 1
set marker(font) helvetica
set marker(font,size) 10
set marker(font,style) normal
set marker(format) ds9
set marker(system) physical
set marker(sky) fk5
set marker(skyformat) degrees
set marker(strip) 0
set marker(maxdialog) 10

# Default Marker Variables
set marker(circle,radius) 20
set marker(ellipse,radius1) 40
set marker(ellipse,radius2) 20
set marker(box,width) 80
set marker(box,height) 40
set marker(projection,thick) 0
set marker(compass,radius) 40
set marker(annulus,inner) 15
set marker(annulus,outer) 30
set marker(annulus,annuli) 1
set marker(ellipseannulus,radius1) 40
set marker(ellipseannulus,radius2) 20
set marker(ellipseannulus,radius3) 60
set marker(ellipseannulus,annuli) 1
set marker(boxannulus,size1) 40
set marker(boxannulus,size2) 20
set marker(boxannulus,size3) 60
set marker(boxannulus,annuli) 1
set marker(panda,inner) 15
set marker(panda,outer) 30
set marker(panda,annuli) 1
set marker(panda,ang1) 0
set marker(panda,ang2) 360
set marker(panda,angnum) 4

set markerfbox(file)      {ds9.reg}
set markerfbox(directory) {}
set markerfbox(dirslist)  {}
set markerfbox(loadtype)  {}
set markerfbox(pattern)   0
set markerfbox(types) {
    {"Region files"   *.reg}
    {"All files"      *}
}

# WCS Variables
set wcs(align) 0
set wcs(align,system) wcs
set wcs(align,sky) fk5

set wcs(system) wcs
set wcs(sky) fk5
set wcs(skyformat) sexagesimal

# Mosaic Variables
set mosaic(fast) 1

# Colorbar Variables
set colorbar(top) .clrbar
set colorbar(name) {}
set colorbar(view) 1
set colorbar(width) 512
set colorbar(height) 18

set colorbar(map) Grey
set colorbar(invert) 0
set colorbar(count) 1
set colorbar(lock) 0
set colorbar(private) false
set colorbar(lock) 0

set colorbar(speed) 512
set colorbar(mult) 10
set colorbar(area) 0
set colorbar(mode) center

set colorbarfbox(file)      {}
set colorbarfbox(directory) {}
set colorbarfbox(dirslist)  {}
set colorbarfbox(loadtype)  {}
set colorbarfbox(pattern)   0
set colorbarfbox(types) {
    {"SAO Colormap files"   *.sao}
    {"LUT Colormap files"   *.lut}
    {"All files"            *}
}

# Info Panel Variables
set info(view) 1
set info(name) ""
set info(filename) 1
set info(object) 0
set info(minmax) 0

set info(wcs) 1
foreach l {A B C D E F G H I J K L M N O P Q R S T U V W X Y Z} {
    set "info(wcs$l)" 0
}

set info(detector) 0
set info(amplifier) 0
set info(physical) 1
set info(image) 1
set info(frame) 1

# Display Coords Variables
set coord(value) 1
set coord(image) 1
set coord(physical) 0
set coord(amplifier) 0
set coord(detector) 0
set coord(wcs) 1
foreach l {A B C D E F G H I J K L M N O P Q R S T U V W X Y Z} {
    set "coord(wcs$l)" 0
}
set coord(sky) fk5
set coord(skyformat) sexagesimal

# Buttons Variables
set buttons(name) ""
set buttons(view) 1

# Canvas Variables
set canvas(name) ""
set canvas(width) 512
#set canvas(height) 480
set canvas(height) 440
set canvas(auto) 1
set canvas(size) 512

# Panner Variables
set pan(name) ""
set pan(view) 1
set pan(size) 128

set pan(compass,image) 1
set pan(compass,wcs) 1
set pan(compass,wcs,system) wcs
set pan(compass,wcs,sky) fk5

set pan(mode) click
set pan(panzoom,state) 1
set pan(panzoom,last) {0 0}

# Magnifier Variables
set mag(name) ""
set mag(view) 1
set mag(size) 128
set mag(region) 1
set mag(cursor) 1
set mag(zoom) 4

# Graph Variables
set graph(size) 150
set graph(font) "*helvetica-medium-r-*-100-*"
set graph(marginX) 35
set graph(marginY) 25
set graph(xMin) 0
set graph(xMax) 10
set graph(yMin) 1
set graph(yMax) 100
set graph(spacer) ""

set graphHorz(name) ""
set graphHorz(view) 0
set graphHorz(grid) 1
set graphHorz(log) false
blt::vector create graphHorzX graphHorzY

set graphVert(name) ""
set graphVert(view) 0
set graphVert(grid) 1
set graphVert(log) false
blt::vector create graphVertX graphVertY

# Printing Variables
if {$tcl_platform(platform)!= "windows"} {
    set ps(dest) printer
} else {
    set ps(dest) file
}
set ps(cmd) "lp"
set ps(filename) "ds9.ps"
set ps(orient) portrait
set ps(color) rgb
set ps(color2) color
set ps(scale) scaled
set ps(size) letter
set ps(level) 2
set ps(interpolate) 0
set ps(resolution) 72
set ps(width) 8.5
set ps(height) 11

set pssavfbox(file)      {}
set pssavfbox(directory) {}
set pssavfbox(dirslist)  {}
set pssavfbox(loadtype)  {}
set pssavfbox(pattern)   0
set pssavfbox(types) {
    {"Postscript files"   *.ps}
    {"All files"          *}
}

# Tile Variables
set tile(top) .tile
set tile(mb) .tilemb
set tile(mode) grid
set tile(grid,row) 10
set tile(grid,col) 10
set tile(grid,mode) automatic
set tile(grid,gap) 4

# Blink Variables
set blink(interval) 500
set blink(index) -1
set blink(id) 0
set blink(state) 0

# FITS Bin Table Support
set bin(function) sum
set bin(factor) 1
set bin(depth) 1
set bin(buffersize) 1024
set bin(smooth) 0
set bin(smooth,function) gaussian
set bin(smooth,radius) 3

set bindialog(top) .bl
set bindialog(mb) .blmb

# Data Cube Variables
set cube(top) .cube
set cube(mb) .cubemb

set cube(slice) 1
set cube(depth) 1
set cube(id) 0

# PanZoom Variables
set panzoom(top) .pz
set panzoom(mb) .pzmb
set panzoom(system) image
set panzoom(sky) fk5
set panzoom(skyformat) sexagesimal
set panzoom(speed) 512

# Contour Variables
set contour(top) .ct
set contour(mb) .ctmb
set contour(view) 0
set contour(color) green
set contour(width) 1
set contour(smooth) 4
set contour(levels) 5
set contour(scale) linear
set contour(method) smooth
set contour(low) ""
set contour(high) ""
set contour(copy) ""

set contourfbox(file)      {ds9.con}
set contourfbox(directory) {}
set contourfbox(dirslist)  {}
set contourfbox(loadtype)  {}
set contourfbox(pattern)   0
set contourfbox(types) {
    {"Contour files"   *.con}
    {"All files"       *}
}

set contourlevfbox(file)      {ds9.lev}
set contourlevfbox(directory) {}
set contourlevfbox(dirslist)  {}
set contourlevfbox(loadtype)  {}
set contourlevfbox(pattern)   0
set contourlevfbox(types) {
    {"Contour Level files"   *.lev}
    {"All files"             *}
}

# Coordinate Grid
set grid(top) .grid
set grid(mb) .gridmb
set grid(view) 0

set grid(system) wcs
set grid(sky) fk5
set grid(skyformat) sexagesimal

set grid(type) analysis
set grid(type,numlab) interior

set grid(grid) 1
set grid(grid,color) 4
set grid(grid,width) 0
set grid(grid,style) 0

set grid(axes) 1
set grid(axes,color) 2
set grid(axes,width) 0
set grid(axes,style) 0

set grid(tick) 1
set grid(tick,color) 1
set grid(tick,width) 0
set grid(tick,style) 0

set grid(border) 1
set grid(border,color) 4
set grid(border,width) 0
set grid(border,style) 0

set grid(labelling) interior
set grid(labelup) 0
set grid(edge,1) bottom
set grid(edge,2) left

set grid(numlab) 1
set grid(numlab,font) helvetica
set grid(numlab,style) bold
set grid(numlab,size) 10
set grid(numlab,color) 3
set grid(numlab,gap1) {}
set grid(numlab,gap2) {}

set grid(title) 1
set grid(title,text) ""
set grid(title,def) 1
set grid(title,gap) {}
set grid(title,font) helvetica
set grid(title,style) bold
set grid(title,size) 12
set grid(title,color) 0

set grid(textlab) 1
set grid(label1) ""
set grid(textlab,def1) 1
set grid(gap1) ""
set grid(gapunit1) ""
set grid(label2) ""
set grid(textlab,def2) 1
set grid(gap2) ""
set grid(gapunit2) ""
set grid(textlab,font) helvetica
set grid(textlab,style) bold
set grid(textlab,size) 10
set grid(textlab,color) 0
set grid(textlab,gap1) {}
set grid(textlab,gap2) {}

set gridfbox(file)      {ds9.grd}
set gridfbox(directory) {}
set gridfbox(dirslist)  {}
set gridfbox(loadtype)  {}
set gridfbox(pattern)   0
set gridfbox(types) {
    {"Grid files"   *.grd}
    {"All files"    *}
}

# Examine Variables
set examine(mode) new
set examine(zoom) 4

# IIS Variables
set iis(state) 0
set iis(cursor) 0
set iis(abort) 0
set iis(ififo) "/dev/imt1i"
set iis(ofifo) "/dev/imt1o"
set iis(port) 5137
set iis(unix) "/tmp/.IMT%d"
set iis(width) 512
set iis(height) 512
set iis(x) -1
set iis(y) -1
set iis(frame) {}

# Name Server Variables
set nsvr(top) .nsvr
set nsvr(mb) .nsvrmb
set nsvr(server) simbad-sao
set nsvr(system) wcs
set nsvr(sky) fk5
set nsvr(skyformat) degrees
set nsvr(name) {}
set nsvr(x) {}
set nsvr(y) {}
set nsvr(state) 0

# DSS Variables
set dss(top) .dss
set dss(mb) .dssmb
set dss(server) sao
set dss(system) wcs
set dss(sky) fk5
set dss(skyformat) arcmin
set dss(name) {}
set dss(x) {}
set dss(y) {}
set dss(width) 15
set dss(height) 15
set dss(survey) dss
set dss(frame) new
set dss(save) 0
set dss(fn) {}
set dss(state) 0
set dss(updatesize) 1

set dssfbox(file)      {dss.fits}
set dssfbox(directory) {}
set dssfbox(dirslist)  {}
set dssfbox(loadtype)  {}
set dssfbox(pattern)   0
set dssfbox(types) {
    {"Fits files"              *.fits}
    {"Fits files"              *.fts }
    {"Fits files"              *.fit }
    {"Compressed FITS files"   *.fitz }
    {"Compressed FITS files"   *.fits.gz }
    {"Compressed FITS files"   *.fits.Z }
    {"All files"               *}
}

# VO Variables
set vo(top) .vo
set vo(mb) .vomb
set vo(timeout) 10000
set vo(state) 0
set vo(server) {http://hea-www.harvard.edu/RD/vo/list.txt}
set vo(server,title) {}
set vo(server,host) {}
set vo(server,url) {}
set vo(server,button) {}
set vo(hv) 1

# Analysis Variables
set analysis(menu,count) 0
set analysis(menu,hmenu,count) 0
set analysis(menu,hmenu) {}
set analysis(bind,count) 0
set analysis(param,count) 0
set analysis(param,seq) 0
set analysis(log) 0

set analysisfbox(file)      {}
set analysisfbox(directory) {}
set analysisfbox(dirslist)  {}
set analysisfbox(loadtype)  {}
set analysisfbox(pattern)   0
set analysisfbox(types) {
    {"Analysis files"   *.ans}
    {"All files"        *}
}

set apdatafbox(file)      {ds9.dat}
set apdatafbox(directory) {}
set apdatafbox(dirslist)  {}
set apdatafbox(loadtype)  {}
set apdatafbox(pattern)   0
set apdatafbox(types) {
    {"Plot Data files"   *.dat}
    {"All files"         *}
}

set apconfigfbox(file)      {ds9.plt}
set apconfigfbox(directory) {}
set apconfigfbox(dirslist)  {}
set apconfigfbox(loadtype)  {}
set apconfigfbox(pattern)   0
set apconfigfbox(types) {
    {"Plot Config files"   *.plt}
    {"All files"           *}
}

# Text Variables
set text(search) {}

set textfbox(file)      {ds9.txt}
set textfbox(directory) {}
set textfbox(dirslist)  {}
set textfbox(loadtype)  {}
set textfbox(pattern)   0
set textfbox(types) {
    {"Text files"   *.txt}
    {"All files"    *}
}

# TCL Variables
set tclfbox(file)      {}
set tclfbox(directory) {}
set tclfbox(dirslist)  {}
set tclfbox(loadtype)  {}
set tclfbox(pattern)   0
set tclfbox(types) {
    {"TCL files"   *.tcl}
    {"All files"   *}
}

# HTTP Variables
set http(last) {}
set http(proxy) 0
set http(proxy,host) {}
set http(proxy,port) {}

# HV Variables
set hv(timeout) 20000
set hv(incr) 0
foreach n {1 2 3 4} {
    set hv(archive,menu,$n) {}
    set hv(archive,url,$n) {}
}

set hvfitsfbox(file)      {hv.fits}
set hvfitsfbox(directory) {}
set hvfitsfbox(dirslist)  {}
set hvfitsfbox(loadtype)  {}
set hvfitsfbox(pattern)   0
set hvfitsfbox(types) {
    {"Fits files"              *.fits}
    {"Fits files"              *.fts }
    {"Fits files"              *.fit }
    {"Compressed FITS files"   *.fitz }
    {"Compressed FITS files"   *.fits.gz }
    {"Compressed FITS files"   *.fits.Z }
    {"All files"               *}
}

set hvhtmlfbox(file)      {}
set hvhtmlfbox(directory) {}
set hvhtmlfbox(dirslist)  {}
set hvhtmlfbox(loadtype)  {}
set hvhtmlfbox(pattern)   0
set hvhtmlfbox(types) {
    {"HTML files"   *.html}
    {"HTML files"   *.htm }
    {"All files"    *}
}

set hvformfbox(file)      {}
set hvformfbox(directory) {}
set hvformfbox(dirslist)  {}
set hvformfbox(loadtype)  {}
set hvformfbox(pattern)   0
set hvformfbox(types) {
    {"All files"     *}
}

set hvsavefbox(file)      {}
set hvsavefbox(directory) {}
set hvsavefbox(dirslist)  {}
set hvsavefbox(loadtype)  {}
set hvsavefbox(pattern)   0
set hvsavefbox(types) {
    {"All files"     *}
}

# SaveAs Variables
set saveas(device) jpeg
set saveas(jpeg,qf) 75
set saveas(resolution) 72
set saveas(base) "ds9"

set saveasfbox(file)      {}
set saveasfbox(directory) {}
set saveasfbox(dirslist)  {}
set saveasfbox(loadtype)  {}
set saveasfbox(pattern)   0
set saveasfbox(types) {
    {"JPEG files"   *.jpg}
    {"TIFF files"   *.tif }
    {"PNG files"    *.png }
    {"PPM files"    *.ppm }
    {"All files"    *}
}

# Plot Tool Variables
set ap(unique,id) 0
set ap(xpa) {}
set ap(tt) "ap"

# Preference Variables

# init files
set prefs(file) "~/.$ds9(app).prf"
set prefs(file,alt) "~/.$ds9(app).prefs"
set prefs(file,local) "/usr/local/lib/$ds9(app).prf"
set prefs(file,opt)  "/opt/local/lib/$ds9(app).prf"
set prefs(startup) "~/.$ds9(app).ini"
set prefs(startup,alt) "~/.$ds9(app)"

if {$tcl_platform(platform) == "windows"} {
    set prefs(file) ".$ds9(app).prf"
    set prefs(file,alt) "$ds9(app).prf"
    set prefs(startup) ".$ds9(app).ini"
    set prefs(startup,alt) "$ds9(app).ini"

    if {[info exists env(TERM)]} {
	if {$env(TERM) == "cygwin"} {
	    set prefs(file) "~/.$ds9(app).prf"
	    set prefs(file,alt) "~/$ds9(app).prf"
	    set prefs(startup) "~/.$ds9(app).ini"
	    set prefs(startup,alt) "~/$ds9(app).ini"
	}
    }
}

set prefs(font) default
set prefs(font,size) 10
set prefs(font,style) normal
set prefs(wcs,system) $wcs(system)
set prefs(wcs,sky) $wcs(sky)
set prefs(wcs,skyformat) $wcs(skyformat)
set prefs(ds9,mode) $ds9(mode)
set prefs(blink,interval) $blink(interval)
set prefs(tile,mode) $tile(mode)
set prefs(bin,function) $bin(function)
set prefs(bin,factor) $bin(factor)
set prefs(bin,depth) $bin(depth)
set prefs(bin,buffersize) $bin(buffersize)
set prefs(bin,smooth) $bin(smooth)
set prefs(bin,smooth,function) $bin(smooth,function)
set prefs(wcs,align) $wcs(align)
set prefs(wcs,align,system) $wcs(align,system)
set prefs(wcs,align,sky) $wcs(align,sky)
set prefs(zoom) $current(zoom)
set prefs(orient) $current(orient)
set prefs(rotate) $current(rotate)
set prefs(minmax,mode) $minmax(mode)
set prefs(minmax,sample) $minmax(sample)
set prefs(scale) $scale(type)
set prefs(scale,mode) $scale(mode)
set prefs(scale,scope) $scale(scope)
set prefs(scale,datasec) $scale(datasec)
set prefs(colorbar,map) $colorbar(map)
set prefs(colorbar,invert) $colorbar(invert)
set prefs(contour,method) $contour(method)
set prefs(contour,color) $contour(color)
set prefs(contour,width) $contour(width)
set prefs(analysis,log) $analysis(log)
set prefs(analysis,user) $ds9(analysis,user)
set prefs(analysis,user2) $ds9(analysis,user2)
set prefs(analysis,user3) $ds9(analysis,user3)
set prefs(analysis,user4) $ds9(analysis,user4)

# Marker Prefs
set prefs(marker,shape) $marker(shape)
set prefs(marker,color) $marker(color)
set prefs(marker,width) $marker(width)
set prefs(marker,edit) $marker(edit)
set prefs(marker,move) $marker(move)
set prefs(marker,rotate) $marker(rotate)
set prefs(marker,delete) $marker(delete)
set prefs(marker,fixed) $marker(fixed)
set prefs(marker,include) $marker(include)
set prefs(marker,source) $marker(source)
set prefs(marker,font) $marker(font)
set prefs(marker,font,size) $marker(font,size)
set prefs(marker,font,style) $marker(font,style)
set prefs(marker,format) $marker(format)
set prefs(marker,strip) $marker(strip)
set prefs(marker,system) $marker(system)
set prefs(marker,sky) $marker(sky)
set prefs(marker,skyformat) $marker(skyformat)

# Page Setup Prefs
set prefs(ps,orient) $ps(orient)
set prefs(ps,scale) $ps(scale)
set prefs(ps,size) $ps(size)
set prefs(ps,width) $ps(width)
set prefs(ps,height) $ps(height)

# VO Prefs
set prefs(vo,hv) $vo(hv)

# Analysis Plot prefs
set prefs(ap,unique) 1

set prefs(ap,grid) 1
set prefs(ap,grid,log) linearlinear

set prefs(ap,discrete) 0
set prefs(ap,discrete,symbol) circle
set prefs(ap,discrete,color) red

set prefs(ap,linear) 1
set prefs(ap,linear,width) 1
set prefs(ap,linear,color) black
set prefs(ap,linear,dash) " "

set prefs(ap,step) 0
set prefs(ap,step,width) 1
set prefs(ap,step,color) black
set prefs(ap,step,dash) " "

set prefs(ap,quadratic) 0
set prefs(ap,quadratic,width) 1
set prefs(ap,quadratic,color) black
set prefs(ap,quadratic,dash) " "

set prefs(ap,error) 1
set prefs(ap,error,width) 1
set prefs(ap,error,color) red
set prefs(ap,error,style) 0

set prefs(ap,titleFont) helvetica
set prefs(ap,titleSize) 12
set prefs(ap,titleStyle) normal

set prefs(ap,textlabFont) helvetica
set prefs(ap,textlabSize) 10
set prefs(ap,textlabStyle) normal

set prefs(ap,numlabFont) helvetica
set prefs(ap,numlabSize) 10
set prefs(ap,numlabStyle) normal

# set default font
if {$prefs(font) != "default"} {
    option add *font "$prefs(font) $prefs(font,size) $prefs(font,style)"
}

# Load any preferences here, before we do any real work
LoadPrefs

# Toplevel-- Start here.
# We want to withdraw the window til everything is ready to go

wm withdraw $ds9(top)

set t "SAOImage $ds9(title)"
wm title $ds9(top) "$t"
wm iconname $ds9(top) "$t"
wm protocol $ds9(top) WM_DELETE_WINDOW QuitDS9
unset t

# This solves this problem:
# 1. tk no longer changes the vert size, thus generating two update sequences
# wm geometry $ds9(top) 750x749
# wm minsize $ds9(top) 640 480
$ds9(top) configure -menu $ds9(mb)

# we need to set certain variables for color before anything else.
# also, set the title
ProcessCommandLineFirst

set ds9(visual) [winfo visual .]
set ds9(depth) [winfo depth .]

if {$tcl_platform(platform)== "windows"} {
    if {$ds9(depth)==32} {
	set ds9(depth) 24
    }
}

if {$ds9(depth)==15} {
    set ds9(depth) 16
}

switch $ds9(visual)$ds9(depth) {
    pseudocolor8 {}
    truecolor8 {}
    truecolor16 {}
    truecolor24 {}
    default {BadVisualError}
}

# create our main frame
frame $ds9(main)

# we want the canvas to always to grow to consume all available space
grid rowconfigure $ds9(main) 2 -weight 1
grid columnconfigure $ds9(main) 0 -weight 1
pack $ds9(main) -fill both -expand true

# Create Colorbar-- Create this first, so in case of a private colormap,
# gui colors will be allocated in the new colormap, not the default colormap
CreateColorbar

# Create Canvas
set canvas(name) [canvas $ds9(main).image -width $canvas(width) \
		      -height $canvas(height) -bd 2 -relief groove \
		      -highlightthickness 0 -insertofftime 0]

# Create other parts of the display
CreateMenuBar
CreateInfoPanel
CreateButtons
CreateGraphs

# Make sure that the wm knows when to swap in the colormap (if needed)
wm colormapwindows . "$ds9(main) $canvas(name)"

# Initialize the display
InitColorbar
InitPanner

# Set our current state of things
CreateColorMenus
UpdateInfoPanel
ChangeMode
LayoutView

# We need this, cause we have SetWatchCursor turned off, hence no idletasks
# for some reason, CreateFrame needs to have an idletasks before 
# creating a new frame

global debug
if {$debug(tcl,idletasks)} {
    puts "ds9.tcl"
}
update idletasks
CreateFrame

# ok, ready to show the window
wm deiconify $ds9(top)

# we do this now, so configure events doesn't run LayoutFrames
InitCanvas

# Init external File Formats
# we want this before processing the command line
InitExternalFile

# Init analysis file formats
InitAnalysisFile

# Configure HTTP
ConfigHTTP

# and process any command line items
# we want to see something before any fits files are loaded
ProcessCommandLine

# Initialize IIS
# after command line options to set port/fifo/unix...
catch {IISInit}

# Initialize XPA
if {$ds9(xpa)} {
    if {[info exists env(XPA_METHOD)]} {
	if {$env(XPA_METHOD) == "local"} {
	    catch {InitXPA}
	} else {
	    if {![checkdns {} 5]} {
		catch {InitXPA}
	    } else {
	      Error "Unable to determine network configuration, disabling XPA"
	    }
	}
    } else {
	if {![checkdns {} 5]} {
	    catch {InitXPA}
	} else {
	    Error "Unable to determine network configuration, disabling XPA"
	}
    }
}

# and finally, process any command line options that are dependant on xpa
ProcessCommandLineXPA

# Load other user tcl code
if {[file exists "$prefs(startup)"]} {
    catch {source "$prefs(startup)"}
} elseif {[file exists "$prefs(startup,alt)"]} {
    catch {source "$prefs(startup,alt)"}
}

# and finally, execute code that prefs may have changed
UpdatePrefsMenu

ChangeImageCompass
ChangeWCSCompass

ChangeMagnifierRegion
ChangeMagnifierCursor
MagnifierZoomTo

# ok, we're done
set ds9(init) 0
