/*
 *
 * gcat -- cat-like program using gio
 *
 */

/*
 *      Copyrights:
 *
 *    	      Copyright (c) 2000 Smithsonian Astrophysical Observatory
 *
 *        Permission to use, copy, modify, distribute, and  sell  this
 *        software  and  its  documentation  for any purpose is hereby
 *        granted without  fee,  provided  that  the  above  copyright
 *        notice  appear  in  all  copies and that both that copyright
 *        notice and this permission notice appear in supporting docu-
 *        mentation,  and  that  the  name  of the  Smithsonian Astro-
 *	  physical Observatory not be used in advertising or publicity
 *  	  pertaining to distribution of the software without specific,
 *	  written  prior  permission.   The Smithsonian  Astrophysical 
 *        Observatory makes no representations about  the  suitability
 *	  of  this  software for any purpose.  It is provided  "as is"
 *	  without express or implied warranty.
 *        THE  SMITHSONIAN  ASTROPHYSICAL  OBSERVATORY  DISCLAIMS  ALL
 *	  WARRANTIES  WITH  REGARD  TO  THIS  SOFTWARE,  INCLUDING ALL
 *	  IMPLIED  WARRANTIES  OF  MERCHANTABILITY AND FITNESS, IN  NO
 *	  EVENT SHALL THE  SMITHSONIAN  ASTROPHYSICAL  OBSERVATORY  BE 
 *	  LIABLE FOR  ANY SPECIAL, INDIRECT  OR  CONSEQUENTIAL DAMAGES
 *	  OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS  OF USE,  DATA
 *	  OR  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 *	  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
 *	  THE  USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <gio.h>

#define GBUFSIZE 8192

#ifdef ANSI_FUNC
int 
main (int argc, char **argv)
#else
main(argc, argv)
     int argc;
     char **argv;
#endif
{
  int got;
  char  buf[GBUFSIZE];
  char *iname;
  char *oname;
  GIO igio;
  GIO ogio;

  /* exit on gio errors */
  setgerror(2);

  /* process arguments */
  switch(argc){
  case 1:
    iname = "stdin";
    oname = "stdout";
    break;
  case 3:
    iname = argv[1];
    oname = argv[2];
    break;
  default:
    gerror(stderr, "%s [iname] [oname]\n", argv[0]);
    exit(1);
  }

  /* open the input and output files */
  if( !(igio = gopen(iname, "r")) )
    gerror(stderr, "can't gopen input file: %s\n", iname);
  if( !(ogio = gopen(oname, "w")) )
    gerror(stderr, "can't gopen output file: %s\n", iname);

  /* read/write all bytes */
  while( 1 ){
    got = gread(igio, buf, sizeof(char), GBUFSIZE);
    if( !got )
      break;
    else if( got < 0 ){
      gerror(stderr, "can't read input file\n");
      break;
    }
    else
      gwrite(ogio,  buf, sizeof(char), got);
  }

  /* close files */
  gclose(igio);
  gclose(ogio);
}
